/**
 * @brief           Application Configuration functions
 * @file            appcfg.h
 * @author          <a href="www.modtronix.com">Modtronix Engineering</a>
 * @dependencies    none
 * @compiler        MPLAB C18 v2.10 or higher <br>
 *                  HITECH PICC-18 V8.35PL3 or higher
 * @ingroup         mod_conf_appconfig
 *
 * @section description Description
 **********************************
 * For details on application configuration data, see @ref mod_conf_appconfig
 * "Application Configuration" module!
 *
 *
 * @subsection appcfg_conf Configuration
 *****************************************
 * The following defines are used to configure this module, and should be placed
 * in the projdefs.h (or similar) file.
 * For details, see @ref mod_conf_projdefs "Project Configuration".
 * To configure the module, the required defines should be uncommended, and the rest commented out.
 @code
 //*******************************************************
 //----------------- Appcfg Configuration -----------------
 //********************************************************
 //Define if this application has an 8 bit ADC converter, 10 bit ADC converter or none at all.
 //#define APP_USE_ADC8
 //#define APP_USE_ADC10

 //Define buffer space that is reserved for ADC conversion values. For a 8 bit ADC converted, 1 byte
 //is reserved per channel. For an 10 bit ADC, 2 bytes are reserved per channel.
 //#define ADC_CHANNELS 12
 @endcode
 *********************************************************************/

 /*********************************************************************
 * Software License Agreement
 *
 * The software supplied herewith is owned by Modtronix Engineering, and is
 * protected under applicable copyright laws. The software supplied herewith is
 * intended and supplied to you, the Company customer, for use solely and
 * exclusively on products manufactured by Modtronix Engineering. The code may
 * be modified and can be used free of charge for commercial and non commercial
 * applications. All rights are reserved. Any use in violation of the foregoing
 * restrictions may subject the user to criminal sanctions under applicable laws,
 * as well as to civil liability for the breach of the terms and conditions of this license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES, WHETHER EXPRESS,
 * IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
 * COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 **********************************************************************
 * File History
 *
 * 2004-10-19, David Hosken (DH):
 *    - Initial version
 *********************************************************************/


/////////////////////////////////////////////////
//  Application Configuration - documentation module
 /**
 * @defgroup mod_conf_appconfig Application Configuration
 * @ingroup mod_conf
 * The application configuration is stored in the PIC microcontroller's internal
 * EEPROM in the first 256 bytes of data. The structure of the data is defined
 * in the @ref APP_CONFIG_ENUM enumeration.<br>
 * The device's default configuration is set in the eeconfigArray[] array in the
 * appcfg.c file. This array will define the values for the entire APP_CONFIG_ENUM, which
 * contains all application configuration. At compilation this data is added to the hex
 * file, which is then written to the PIC's internal EEPROM at programming. Some
 * programmers will have the options during programming not to program the PIC's internal
 * EEPROM memory. It is important to enable the programming of the internal EEPROM to
 * have this default device configuration written to the EEPROM.
 *********************************************************************/

#ifndef APPCFG_H
#define APPCFG_H

#include "net\ip.h"


/////////////////////////////////////////////////
//Do some checks on defines
#if ((ADC_CHANNELS < 0) || (ADC_CHANNELS > 12))
#error "ADC_CHANNELS has been assigned an illegal value!"
#endif

#if (ADC_CHANNELS > 0)
    #if !defined(APP_USE_ADC8) && !defined(APP_USE_ADC10)
    #define APP_USE_ADC8
    #endif
#endif

#if (ADC_CHANNELS > 0)
    #if defined(APP_USE_ADC8) && defined(APP_USE_ADC10)
    #error "APP_USE_ADC8 and APP_USE_ADC10 can not both be defined!"
    #endif
#endif

/////////////////////////////////////////////////
//CPU configuration stuff

#define BAUD_300        0
#define BAUD_1200       1
#define BAUD_2400       2
#define BAUD_4800       3
#define BAUD_9600       4
#define BAUD_19200      5
#define BAUD_38400      6
#define BAUD_57600      7
#define BAUD_115200     8
#define BAUD_230400     9
#define BAUD_460800     10
#define BAUD_921600     11


//Define maximum username length. This is the lengh of the characters + NULL terminating char. Thus, a value
//of 9 = 8 char string + NULL
#define APPCFG_USERNAME_LEN 9

/**
 * Structure for storing application configuration data that needs fast access.
 * This structure is created in the processors internal RAM, and is initialized with
 * the contents of the corresponding bytes in EEPROM at startup. Whenever the application
 * needs to access any of these variables, it will read them straight from this RAM structure
 * in stead of reading them from the EEPROM.
 */
typedef struct _APP_CONFIG
{
    IP_ADDR     MyIPAddr;        //4 bytes
    MAC_ADDR    MyMACAddr;        //6 bytes
    IP_ADDR     MyMask;            //4 bytes
    IP_ADDR     MyGateway;        //4 bytes
} APP_CONFIG;

/**
 * Mask for bits in APPCFG_NETFLAGS config byte.
 */
#define APPCFG_NETFLAGS_DHCP    0x01

#if defined(BRD_SBC65EC) || defined(BRD_SBC68EC)
//@{
/**
 * Mask for bits in APPCFG_SYSFLAGS config byte
 */
#define APPCFG_SYSFLAGS_PLLON   0x01
#define APPCFG_SYSFLAGS_BLINKB6 0x02
//@}
#endif

/**
 * Mask for bits in APPCFG_CMD_FLAGS config byte.
 */
#define APPCFG_CMDFLAGS_ENABLE    0x01
#define APPCFG_CMDFLAGS_AUTH_UDP  0x02


/**
 * Mask for bits in APPCFG_WEB_FLAGS config byte.
 */
#define APPCFG_WEBFLAGS_AUTH_X      0x01  /* Authentication required for all files that start with 'X' character */
#define APPCFG_WEBFLAGS_AUTH_ALL    0x02  /* Authentication required for all */
#define APPCFG_WEBFLAGS_AUTH_GET    0x04  /* Authentication required for all pages with GET Methods */
#define APPCFG_WEBFLAGS_AUTH_DYN    0x08  /* Authentication required for all Dynamic files */
#define APPCFG_WEBFLAGS_AUTH_CGI    0x10  /* Authentication required for all CGI files */
#define APPCFG_WEBFLAGS_AUTH_SECTAG 0x20  /* Authentication required for all Secure Tags */

/**
 * Mask for bits in APPCFG_WEB_FLAGS2 config byte.
 */
#define APPCFG_WEBFLAGS_PARSE_HTM  0x01
#define APPCFG_WEBFLAGS_PARSE_JS   0x02

/**
 * Define this if APP_CONFIG is smaller then 256 bytes. When this is defined
 * the code knows that it only need 1 byte to address all configuration bytes. This
 * produces smaller and faster code.
 */
#define APPCFG_SMALLER_256


/**
 * Main application config structure. This structure lives in the internal
 * EEPROM, starting at address 0.<br>
 * <b>!!! IMPORTANT - Only add to end the of this structure!!!</b><br>
 * This structure can NOT get larger then 256 bytes. If it does, some code will have to be
 * changed!
 */
enum APP_CONFIG_ENUM {
    APPCFG_IP0 = 0,                 ///< Offset = 0 = 0x00<br>Our IP address. First entry is MSB byte of IP address, for example in "192.168.1.0" it will be 192.
    APPCFG_IP1,                     ///< Offset = 1 = 0x01
    APPCFG_IP2,                     ///< Offset = 2 = 0x02
    APPCFG_IP3,                     ///< Offset = 3 = 0x03

    /**< <hr>
     * Our <b>MAC address</b>. First entry is MSB byte of MAC address
     */
    APPCFG_MAC0,                    ///< Offset = 4 = 0x04
    APPCFG_MAC1,                    ///< Offset = 5 = 0x05
    APPCFG_MAC2,                    ///< Offset = 6 = 0x06
    APPCFG_MAC3,                    ///< Offset = 7 = 0x07
    APPCFG_MAC4,                    ///< Offset = 8 = 0x08
    APPCFG_MAC5,                    ///< Offset = 9 = 0x09

    /**< <hr>
     * Our <b>Subnet Mask</b>. First entry is MSB of MASK address, for example if mask is 255.0.0.0 it will be 255
     */
    APPCFG_MASK0,                   ///< Offset = 10 = 0x0A
    APPCFG_MASK1,                   ///< Offset = 11 = 0x0B
    APPCFG_MASK2,                   ///< Offset = 12 = 0x0C
    APPCFG_MASK3,                   ///< Offset = 13 = 0x0D

    /**< <hr>
     * Our <b>Gateway address</b>. First entry is MSB byte of IP address, for example in "192.168.1.254" it will be 192
     */
    APPCFG_GATEWAY0,                ///< Offset = 14 = 0x0E
    APPCFG_GATEWAY1,                ///< Offset = 15 = 0x0F
    APPCFG_GATEWAY2,                ///< Offset = 16 = 0x10
    APPCFG_GATEWAY3,                ///< Offset = 17 = 0x11

    /**< <hr>
     * <b>Bootloader Startup Delay</b>. This is how long the bootloader will run after reset, value from 0-n. Time = 0.8sec x n
     */
    APPCFG_WAIT4BL,                 ///< Offset = 18 = 0x12

    /**< <hr>
     * If 0, the FLASH has not been programmed yet, or does NOT contain a valid program. In this case we can
     * not exit bootloader mode!
     */
    APPCFG_FLASH_HAS_PROGRAM,       ///< Offset = 19 = 0x13

    /**< <hr>
     * <b>Time to delay on startup</b>(n x 50ms), and give chance to press key and enter serial config. If 0, skip
     */
    APPCFG_STARTUP_SER_DLY,         ///< Offset = 20 = 0x14

    /**< <hr><b>System Flags</b>
     *  - Bit 0: 4 x PLL enable flag
     *  - Bit 1: Blink led on B6
     */
    APPCFG_SYSFLAGS,                ///< Offset = 21 = 0x15

    /**< <hr><b>Network Flags</b>
     *  - Bit 0: DHCP enable flag
     */
    APPCFG_NETFLAGS,                ///< Offset = 22 = 0x16

    /**< <hr><b>SMTP Server address</b> */
    APPCFG_SMTP_SRVR_ADR0,          ///< Offset = 23 = 0x17
    APPCFG_SMTP_SRVR_ADR1,          ///< Offset = 24 = 0x18
    APPCFG_SMTP_SRVR_ADR2,          ///< Offset = 25 = 0x19
    APPCFG_SMTP_SRVR_ADR3,          ///< Offset = 26 = 0x1A

    /**< <hr><b>TFTP Server address</b> */
    APPCFG_TFTP_SRVR_ADR0,          ///< Offset = 27 = 0x1B
    APPCFG_TFTP_SRVR_ADR1,          ///< Offset = 28 = 0x1C
    APPCFG_TFTP_SRVR_ADR2,          ///< Offset = 29 = 0x1D
    APPCFG_TFTP_SRVR_ADR3,          ///< Offset = 30 = 0x1E

    /**< <hr><b>USART1</b> Configuration. Baud rate will be a BAUD_XXX define */
    APPCFG_USART1_BAUD,             ///< Offset = 31 = 0x1F
    APPCFG_USART1_CFG,              ///< Offset = 32 = 0x20

    /**< <hr><b>USART2</b> Configuration. Baud rate will be a BAUD_XXX define */
    APPCFG_USART2_BAUD,             ///< Offset = 33 = 0x21
    APPCFG_USART2_CFG,              ///< Offset = 34 = 0x22

    /**< <hr><hr><b>Port Direction</b> configuration */
    APPCFG_TRISA,                   ///< Offset = 35 = 0x23
    APPCFG_TRISB,                   ///< Offset = 36 = 0x24
    APPCFG_TRISC,                   ///< Offset = 37 = 0x25
    APPCFG_TRISD,                   ///< Offset = 38 = 0x26
    APPCFG_TRISE,                   ///< Offset = 39 = 0x27
    APPCFG_TRISF,                   ///< Offset = 40 = 0x28
    APPCFG_TRISG,                   ///< Offset = 41 = 0x29

    /**< <hr><b>Port Default Values</b> */
    APPCFG_PORTA,                   ///< Offset = 42 = 0x2A
    APPCFG_PORTB,                   ///< Offset = 43 = 0x2B
    APPCFG_PORTC,                   ///< Offset = 44 = 0x2C
    APPCFG_PORTD,                   ///< Offset = 45 = 0x2D
    APPCFG_PORTE,                   ///< Offset = 46 = 0x2E
    APPCFG_PORTF,                   ///< Offset = 47 = 0x2F
    APPCFG_PORTG,                   ///< Offset = 48 = 0x30

    /**< <hr><b>ADC</b> configuration */
    APPCFG_ADCON1,                  ///< Offset = 49 = 0x31
    APPCFG_ADCON0_2,                ///< Offset = 50 = 0x32

    /**< <hr><b>Serial Number</b> */
    APPCFG_SERIAL_NUMBER0,          ///< Offset = 51 = 0x33
    APPCFG_SERIAL_NUMBER1,          ///< Offset = 52 = 0x34

    /**< <hr><b>Username</b> - Is a null terminated string. Max 8 char + null termination char*/
    APPCFG_USERNAME0,               ///< Offset = 53 = 0x35
    APPCFG_USERNAME1,               ///< Offset = 54 = 0x36
    APPCFG_USERNAME2,               ///< Offset = 55 = 0x37
    APPCFG_USERNAME3,               ///< Offset = 56 = 0x38
    APPCFG_USERNAME4,               ///< Offset = 57 = 0x39
    APPCFG_USERNAME5,               ///< Offset = 58 = 0x3A
    APPCFG_USERNAME6,               ///< Offset = 59 = 0x3B
    APPCFG_USERNAME7,               ///< Offset = 60 = 0x3C
    APPCFG_USERNAME8,               ///< Offset = 61 = 0x3D

    /**< <hr><b>Password</b> - Is a null terminated string. Max 8 char + null termination char*/
    APPCFG_PASSWORD0,               ///< Offset = 62 = 0x3E
    APPCFG_PASSWORD1,               ///< Offset = 63 = 0x3F
    APPCFG_PASSWORD2,               ///< Offset = 64 = 0x40
    APPCFG_PASSWORD3,               ///< Offset = 65 = 0x41
    APPCFG_PASSWORD4,               ///< Offset = 66 = 0x42
    APPCFG_PASSWORD5,               ///< Offset = 67 = 0x43
    APPCFG_PASSWORD6,               ///< Offset = 68 = 0x44
    APPCFG_PASSWORD7,               ///< Offset = 69 = 0x45
    APPCFG_PASSWORD8,               ///< Offset = 70 = 0x46

    /**< <hr><b>NetBIOS Name</b> - Is a 16 character name, all in uppercase */
    APPCFG_NETBIOS0,                ///< Offset = 71 = 0x47
    APPCFG_NETBIOS1,                ///< Offset = 72 = 0x48
    APPCFG_NETBIOS2,                ///< Offset = 73 = 0x49
    APPCFG_NETBIOS3,                ///< Offset = 74 = 0x4A
    APPCFG_NETBIOS4,                ///< Offset = 75 = 0x4B
    APPCFG_NETBIOS5,                ///< Offset = 76 = 0x4C
    APPCFG_NETBIOS6,                ///< Offset = 77 = 0x4D
    APPCFG_NETBIOS7,                ///< Offset = 78 = 0x4E
    APPCFG_NETBIOS8,                ///< Offset = 79 = 0x4F
    APPCFG_NETBIOS9,                ///< Offset = 80 = 0x50
    APPCFG_NETBIOS10,               ///< Offset = 81 = 0x51
    APPCFG_NETBIOS11,               ///< Offset = 82 = 0x52
    APPCFG_NETBIOS12,               ///< Offset = 83 = 0x53
    APPCFG_NETBIOS13,               ///< Offset = 84 = 0x54
    APPCFG_NETBIOS14,               ///< Offset = 85 = 0x55
    APPCFG_NETBIOS15,               ///< Offset = 86 = 0x56

    /**< <hr><b>Primary DNS Server</b>. First entry is MSB byte of IP address, for example in "192.168.1.0" it will be 192 */
    APPCFG_DNS_IP0,                 ///< Offset = 87 = 0x57
    APPCFG_DNS_IP1,                 ///< Offset = 88 = 0x58
    APPCFG_DNS_IP2,                 ///< Offset = 89 = 0x59
    APPCFG_DNS_IP3,                 ///< Offset = 90 = 0x5A

    /**< <hr><b>UDP Command Port</b> */
    APPCFG_CMD_UDPPORT0,            ///< Offset = 91 = 0x5B
    APPCFG_CMD_UDPPORT1,            ///< Offset = 92 = 0x5C

    /**< <hr><b>UDP Command Responce Port</b> */
    APPCFG_CMDRESP_UDPPORT0,        ///< Offset = 93 = 0x5D
    APPCFG_CMDRESP_UDPPORT1,        ///< Offset = 94 = 0x5E

    /**< <hr><b>UDP reserved bytes</b> */
    APPCFG_CMD_RESERVED1,           ///< Offset = 95 = 0x5F
    APPCFG_CMD_RESERVED2,           ///< Offset = 96 = 0x60
    APPCFG_CMD_RESERVED3,           ///< Offset = 97 = 0x61

    /**< <hr><b>Command Flags</b>
     *  - Bit 0: Enable UDP Commands. When set, UDP command are enabled.
     *  - Bit 1: Authentication required for UDP Commands. When set, each UDP command must start with the username
     *    and password, seperated with a ':' character. Following this must be a space, and then the commands.
     */
    APPCFG_CMD_FLAGS,               ///< Offset = 98 = 0x62

    /**< <hr><b>PWM Channel 1-8 Enable</b> Register.
     * 0=off, 1=on. All Off by default. The LSB (bit 0) controls PWM Channel 1.
     */
    APPCFG_PWM_ENABLE,              ///< Offset = 99 = 0x63
    
    /**< <hr>
     *  <table border="0" cellpadding="0" cellspacing="0" width="100%">
     *  <tr><td><b>PWM Frequency</b> for all channels. The frequency is different for 8 bit 10 bit mode! </td></tr>
     *  <tr><td><i>Frequencies when in 10 bit mode:</i></td></tr>
     *  <tr><td class="ind1">- 0 = 2.441 kHz</td></tr>
     *  <tr><td class="ind1">- 1 = 9.766 kHz</td></tr>
     *  <tr><td class="ind1">- 2 and 3 = 39.062 kHz</td></tr>
     *  <tr><td><i>Frequencies when in 8 bit mode:</i></td></tr>
     *  <tr><td class="ind1">- 0 and 1 = 9.766 kHz</td></tr>
     *  <tr><td class="ind1">- 2 = 39.062 kHz</td></tr>
     *  <tr><td class="ind1">- 3 = 156.25 kHz</td></tr>
     *  </table>
     */
    APPCFG_PWM_FREQ,                ///< Offset = 100 = 0x64
    
    /**< <hr><b>PWM Flags:</b>
     *  - Bit 0: Configures for 8 or 10 bit PWM. 0 = 8 bit, 1 = 10 bit.
     */
    APPCFG_PWM_FLAGS,               ///< Offset = 101 = 0x65

    /**< <hr><b>PWM Channel 1</b> (C1) default value (10 bit value) */
    APPCFG_PWM_1L,                  ///< Offset = 102 = 0x66
    APPCFG_PWM_1H,                  ///< Offset = 103 = 0x67

    /**< <hr><b>PWM Channel 2</b> (C2) default value (10 bit value) */
    APPCFG_PWM_2L,                  ///< Offset = 104 = 0x68
    APPCFG_PWM_2H,                  ///< Offset = 105 = 0x69

    /**< <hr><b>PWM Channel 3</b> (G0) default value (10 bit value) */
    APPCFG_PWM_3L,                  ///< Offset = 106 = 0x6A
    APPCFG_PWM_3H,                  ///< Offset = 107 = 0x6B

    /**< <hr><b>PWM Channel 4</b> (G3) default value (10 bit value) */
    APPCFG_PWM_4L,                  ///< Offset = 108 = 0x6C
    APPCFG_PWM_4H,                  ///< Offset = 109 = 0x6D

    /**< <hr><b>PWM Channel 5</b> (G4) default value (10 bit value) */
    APPCFG_PWM_5L,                  ///< Offset = 110 = 0x6E
    APPCFG_PWM_5H,                  ///< Offset = 111 = 0x6F

    /**< <hr><b>Web Server reserved bytes</b> */
    APPCFG_WEB_RESERVED1,           ///< Offset = 112 = 0x70
    APPCFG_WEB_RESERVED2,           ///< Offset = 113 = 0x71

    /**< <hr><b>Web Server Flags</b>
     *  - Bit 0: Authentication required for pages that start with 'x' character.
     *  - Bit 1: Authentication required for ALL pages.
     *  - Bit 2: Authentication required for pages that execute the HTTP Get Method.
     *           When set, HTTP GET Methods are only executed for Authenticated users.
     *  - Bit 3: Authentication required for all Dynamic files.
     *  - Bit 4: Authentication required for all CGI files.
     */
    APPCFG_WEB_FLAGS,               ///< Offset = 115 = 0x72

    /**< <hr><b>Web Server reserved bytes</b> */
    APPCFG_WEB_RESERVED3,           ///< Offset = 112 = 0x73

    /**< <hr><b>HTTP Server Port</b> The default HTTP Server Port is 80 */
    APPCFG_HTTPSRVR_PORTL,        ///< Offset = 116 = 0x74
    APPCFG_HTTPSRVR_PORTH,        ///< Offset = 117 = 0x75
	
	/**< <hr><b>Unused Area</b> */
	APPCFG_EMPTY0,           		///< Offset = 118 = 0x76
	APPCFG_EMPTY1,           		///< Offset = 119 = 0x77
	APPCFG_EMPTY2,           		///< Offset = 120 = 0x78
	APPCFG_EMPTY3,           		///< Offset = 121 = 0x79
	/**< <hr><b>Host TCP port for data upload</b> */
	APPCFG_HTTP_SERVICE_PORTL,        ///< Offset = 122 = 0x7A
	APPCFG_HTTP_SERVICE_PORTH,        ///< Offset = 123 = 0x7B
	
	/**< <hr><b>Host server IP</b> */
	APPCFG_HOST_IP0,           		///< Offset = 124 = 0x7C
	APPCFG_HOST_IP1,           		///< Offset = 125 = 0x7D
	APPCFG_HOST_IP2,           		///< Offset = 126 = 0x7E
	APPCFG_HOST_IP3,           		///< Offset = 127 = 0x7F

   /**< <hr><b>Host Name</b> - Is a 32 character name */
	APPCFG_HOST_NAME0,				///< Offset = 128 = 0x80
	APPCFG_HOST_NAME1,				///< Offset = 129 = 0x81
	APPCFG_HOST_NAME2,				///< Offset = 130 = 0x82
	APPCFG_HOST_NAME3,				///< Offset = 131 = 0x83
	APPCFG_HOST_NAME4,				///< Offset = 132 = 0x84
	APPCFG_HOST_NAME5,				///< Offset = 133 = 0x85
	APPCFG_HOST_NAME6,				///< Offset = 134 = 0x86
	APPCFG_HOST_NAME7,				///< Offset = 135 = 0x87
	APPCFG_HOST_NAME8,				///< Offset = 136 = 0x88
	APPCFG_HOST_NAME9,				///< Offset = 137 = 0x89
	APPCFG_HOST_NAME10,				///< Offset = 138 = 0x8A
	APPCFG_HOST_NAME11,				///< Offset = 139 = 0x8B
	APPCFG_HOST_NAME12,				///< Offset = 140 = 0x8C
	APPCFG_HOST_NAME13,				///< Offset = 141 = 0x8D
	APPCFG_HOST_NAME14,				///< Offset = 142 = 0x8E
	APPCFG_HOST_NAME15,				///< Offset = 143 = 0x8F
	APPCFG_HOST_NAME16,				///< Offset = 144 = 0x90
	APPCFG_HOST_NAME17,				///< Offset = 145 = 0x91
	APPCFG_HOST_NAME18,				///< Offset = 146 = 0x92
	APPCFG_HOST_NAME19,				///< Offset = 147 = 0x93
	APPCFG_HOST_NAME20,				///< Offset = 148 = 0x94
	APPCFG_HOST_NAME21,				///< Offset = 149 = 0x95
	APPCFG_HOST_NAME22,				///< Offset = 150 = 0x96
	APPCFG_HOST_NAME23,				///< Offset = 151 = 0x97
	APPCFG_HOST_NAME24,				///< Offset = 152 = 0x98
	APPCFG_HOST_NAME25,				///< Offset = 153 = 0x99
	APPCFG_HOST_NAME26,				///< Offset = 154 = 0x9A
	APPCFG_HOST_NAME27,				///< Offset = 155 = 0x9B
	APPCFG_HOST_NAME28,				///< Offset = 156 = 0x9C
	APPCFG_HOST_NAME29,				///< Offset = 157 = 0x9D
	APPCFG_HOST_NAME30,				///< Offset = 158 = 0x9E
	APPCFG_HOST_NAME31,				///< Offset = 159 = 0x9F

   /**< <hr><b>HTTP request Path</b> - Is a 32 character name */
	APPCFG_HTTP_REQ_PATH0,			///< Offset = 160 = 0xA0
	APPCFG_HTTP_REQ_PATH1,			///< Offset = 161 = 0xA1
	APPCFG_HTTP_REQ_PATH2,			///< Offset = 162 = 0xA2
	APPCFG_HTTP_REQ_PATH3,			///< Offset = 163 = 0xA3
	APPCFG_HTTP_REQ_PATH4,			///< Offset = 164 = 0xA4
	APPCFG_HTTP_REQ_PATH5,			///< Offset = 165 = 0xA5
	APPCFG_HTTP_REQ_PATH6,			///< Offset = 166 = 0xA6
	APPCFG_HTTP_REQ_PATH7,			///< Offset = 167 = 0xA7
	APPCFG_HTTP_REQ_PATH8,			///< Offset = 168 = 0xA8
	APPCFG_HTTP_REQ_PATH9,			///< Offset = 169 = 0xA9
	APPCFG_HTTP_REQ_PATH10,			///< Offset = 170 = 0xAA
	APPCFG_HTTP_REQ_PATH11,			///< Offset = 171 = 0xAB
	APPCFG_HTTP_REQ_PATH12,			///< Offset = 172 = 0xAC
	APPCFG_HTTP_REQ_PATH13,			///< Offset = 173 = 0xAD
	APPCFG_HTTP_REQ_PATH14,			///< Offset = 174 = 0xAE
	APPCFG_HTTP_REQ_PATH15,			///< Offset = 175 = 0xAF
	APPCFG_HTTP_REQ_PATH16,			///< Offset = 176 = 0xB0
	APPCFG_HTTP_REQ_PATH17,			///< Offset = 177 = 0xB1
	APPCFG_HTTP_REQ_PATH18,			///< Offset = 178 = 0xB2
	APPCFG_HTTP_REQ_PATH19,			///< Offset = 179 = 0xB3
	APPCFG_HTTP_REQ_PATH20,			///< Offset = 180 = 0xB4
	APPCFG_HTTP_REQ_PATH21,			///< Offset = 181 = 0xB5
	APPCFG_HTTP_REQ_PATH22,			///< Offset = 182 = 0xB6
	APPCFG_HTTP_REQ_PATH23,			///< Offset = 183 = 0xB7
	APPCFG_HTTP_REQ_PATH24,			///< Offset = 184 = 0xB8
	APPCFG_HTTP_REQ_PATH25,			///< Offset = 185 = 0xB9
	APPCFG_HTTP_REQ_PATH26,			///< Offset = 186 = 0xBA
	APPCFG_HTTP_REQ_PATH27,			///< Offset = 187 = 0xBB
	APPCFG_HTTP_REQ_PATH28,			///< Offset = 188 = 0xBC
	APPCFG_HTTP_REQ_PATH29,			///< Offset = 189 = 0xBD
	APPCFG_HTTP_REQ_PATH30,			///< Offset = 190 = 0xBE
	APPCFG_HTTP_REQ_PATH31,			///< Offset = 191 = 0xBF
	
	   /**< <hr><b>SNTP Host Name</b> - Is a 32 character name */
	APPCFG_SNTP_NAME0,				///< Offset = 192 = 0xC0
	APPCFG_SNTP_NAME1,				///< Offset = 193 = 0xC1
	APPCFG_SNTP_NAME2,				///< Offset = 194 = 0xC2
	APPCFG_SNTP_NAME3,				///< Offset = 195 = 0xC3
	APPCFG_SNTP_NAME4,				///< Offset = 196 = 0xC4
	APPCFG_SNTP_NAME5,				///< Offset = 197 = 0xC5
	APPCFG_SNTP_NAME6,				///< Offset = 198 = 0xC6
	APPCFG_SNTP_NAME7,				///< Offset = 199 = 0xC7
	APPCFG_SNTP_NAME8,				///< Offset = 200 = 0xC8
	APPCFG_SNTP_NAME9,				///< Offset = 201 = 0xC9
	APPCFG_SNTP_NAME10,				///< Offset = 202 = 0xCA
	APPCFG_SNTP_NAME11,				///< Offset = 203 = 0xCB
	APPCFG_SNTP_NAME12,				///< Offset = 204 = 0xCC
	APPCFG_SNTP_NAME13,				///< Offset = 205 = 0xCD
	APPCFG_SNTP_NAME14,				///< Offset = 206 = 0xCE
	APPCFG_SNTP_NAME15,				///< Offset = 207 = 0xCF
	APPCFG_SNTP_NAME16,				///< Offset = 208 = 0xD0
	APPCFG_SNTP_NAME17,				///< Offset = 209 = 0xD1
	APPCFG_SNTP_NAME18,				///< Offset = 210 = 0xD2
	APPCFG_SNTP_NAME19,				///< Offset = 211 = 0xD3
	APPCFG_SNTP_NAME20,				///< Offset = 212 = 0xD4
	APPCFG_SNTP_NAME21,				///< Offset = 213 = 0xD5
	APPCFG_SNTP_NAME22,				///< Offset = 214 = 0xD6
	APPCFG_SNTP_NAME23,				///< Offset = 215 = 0xD7
	APPCFG_SNTP_NAME24,				///< Offset = 216 = 0xD8
	APPCFG_SNTP_NAME25,				///< Offset = 217 = 0xD9
	APPCFG_SNTP_NAME26,				///< Offset = 218 = 0xDA
	APPCFG_SNTP_NAME27,				///< Offset = 219 = 0xDB
	APPCFG_SNTP_NAME28,				///< Offset = 220 = 0xDC
	APPCFG_SNTP_NAME29,				///< Offset = 221 = 0xDD
	APPCFG_SNTP_NAME30,				///< Offset = 222 = 0xDE
	APPCFG_SNTP_NAME31				///< Offset = 223 = 0xDF
};


/**
 * Second application config structure. Contains configuration for:
 * - SPI, I2C, USART, CAN, USB serial buses
 *This structure lives in the internal
 * EEPROM, starting at address 256.<br>
 * <b>!!! IMPORTANT - Only add to end the of this structure!!!</b><br>
 * This structure can NOT get larger then 256 bytes. If it does, some code will have to be
 * changed!
 */
enum APP_CONFIG2_ENUM {

    /**< Software SPI buses enabled. Bits 0 to 7 indicate which software SPI buses have
    been enabled. */
    APPCFG_SWSPI_ENABLED,           ///< Offset = 0 = 0x00

    /**< Software SPI 1 CLK pin. Bits 4-7 point to port (0=a, 1=b..), and bits 0-3 the bit */
    APPCFG_SWSPI1_CLKPIN,           ///< Offset = 1 = 0x00
    /**< Software SPI 1 SDI pin. Bits 4-7 point to port (0=a, 1=b..), and bits 0-3 the bit */
    APPCFG_SWSPI1_SDIPIN,           ///< Offset = 2 = 0x00
    /**< Software SPI 1 SDO pin. Bits 4-7 point to port (0=a, 1=b..), and bits 0-3 the bit */
    APPCFG_SWSPI1_SDOPIN,           ///< Offset = 3 = 0x00
    /**< Software SPI 1 CS pin. Bits 4-7 point to port (0=a, 1=b..), and bits 0-3 the bit */
    APPCFG_SWSPI1_CSPIN,            ///< Offset = 4 = 0x00
    /**< Software SPI 1 Rx buf. Bits 4-7 = buffer size (0=8, 1=16, .. 5=256), and bits 0-3 the buffer */
    APPCFG_SWSPI1_RXBUF,            ///< Offset = 5 = 0x00
    /**< Software SPI 1 Tx buf. Bits 4-7 = buffer size (0=8, 1=16, .. 5=256), and bits 0-3 the buffer */
    APPCFG_SWSPI1_TXBUF,            ///< Offset = 6 = 0x00
    /**< Software SPI 1 config.
    - Bits 0-1 = SPI Mode<br>
        00 = Mode 0<br>
        01 = Mode 1<br>
        10 = Mode 2<br>
        11 = Mode 3
    - Bits 4-7 = Max clock speed<br>
        0 = Maximum clock speed, no delays<br>
        1 = About 500kbit / second maximum speed<br>
        2 = About 250kbit / second maximum speed<br>
        3 = About 125kbit / second maximum speed<br>
        4 = About 62kbit / second maximum speed<br>
        5 = About 31kbit / second maximum speed<br>
        6 = About 15kbit / second maximum speed<br>
        7 = About 8kbit / second maximum speed<br>
        8 = About 4kbit / second maximum speed<br>
        9 = About 2kbit / second maximum speed<br>
    */
    APPCFG_SWSPI1_CONF            ///< Offset = 7 = 0x00
};


/////////////////////////////////////////////////
//Global variables

//This is used by other stack elements
#ifndef THIS_IS_APPCFG_MODULE

extern APP_CONFIG AppConfig;

/**
 * Global Array containing converted ADC values. This array can be accessed from
 * anywhere to obtain the current value of the ADC inputs. For example, to get
 * the current value of AN2 ADC, use AdcValues[2]
 */
#if defined(APP_USE_ADC8)
extern BYTE AdcValues[ADC_CHANNELS];
#elif defined(APP_USE_ADC10)
extern WORD AdcValues[ADC_CHANNELS];
#endif

#endif


/**
 * @preCondition  fsysInit() is already called.
 */
void appcfgInit(void);

/**
 * Save Application Configuartion data to EEPROM
 */
void appcfgSave(void);

/**
 * Configure USART with AppConfig data
 */
void appcfgUSART(void);

/**
 * Configure the CPU I/O ports
 */
void appcfgCpuIO(void);

/**
 * Configure the CPU I/O ports to have the configured default values
 */
void appcfgCpuIOValues(void);

/**
 * Configure the ADC unit
 */
void appcfgADC(void);

/*
 * Configure the PWM ports with default power up values
 */
void appcfgPWM(void);

/**
 * Sets the PWM frequency to the given value
 *
 * @param freq New frequency, a value from 0-3.
 *             For 8 bit mode:  0 and 1=9.766kHz, 2=39.062kHz, 3=156.25kHz
 *             For 10 bit mode: 0=2.441kHz, 1=9.766kHz, 2 and 3=39.062kHz
 * @param remember If 1, store in EEPROM and remember
 */
void pwmSetFreq(BYTE freq, BOOL remember);

/**
 * Gets a constant value representing the current PWM frequency
 *
 * @return Frequency constant, a value from 0-3.
 *          For 8 bit mode:  1=9.766kHz, 2=39.062kHz, 3=156.25kHz
 *          For 10 bit mode: 0=2.441kHz, 1=9.766kHz, 2=39.062kHz
 */
BYTE pwmGetFreq(void);

/**
 * Return TRUE if the PWM is currently in 10-bit mode
 */
#define pwmIs10BitMode() (PR2 == 255)

/**
 * Return TRUE if the PWM is currently in 10-bit mode
 */
#define pwmIs8BitMode() (PR2 == 63)


/**
 * Set the given PWM channel to the given value
 *
 * @param ch PWM channnel, is a value from 1 to 5
 * @param val The value to set the channel.
 *            - For 8 bit mode, is a value from 0-255
 *            - For 10 bit mode, is a value from 0-1023
 * @param remember If 1, store in EEPROM and remember
 *
 */
void pwmSetValue(BYTE ch, WORD_VAL val, BOOL remember);


/**
 * Get the value of the given PWM channel
 *
 * @param ch PWM channnel, is a value from 1 to 5
 *
 * @return The given PWM channels value
 *
 */
WORD pwmGetValue(BYTE ch);


/**
 * Enables/Disables given PWM channels
 *
 * @param chan Bits 0-4 indicate if PWM channels 1-5 must be enabled or disabled
 * @param remember If 1, store in EEPROM and remember
 */
void pwmEnable(BYTE chan, BOOL remember);

/**
 * Get a specified byte from the APP_CONFIG configuration structure
 *
 * @param offset Contains the offset in the APP_CONFIG structure of the byte to get. Use
 *        the APPCFG_X defines
 *
 * @return Returns the requested byte
 */
BYTE appcfgGetc(BYTE offset);

/**
 * Gets a specified byte from the APP_CONFIG configuration structure
 *
 * @param field The byte to get from the APP_CONFIG structure. For example "MyMask.v[0]"
 *
 * @return Returns the requested byte
 */
#define appcfgGetcAtField(field) appcfgGetc(offsetof(APP_CONFIG, field))

/**
 * Set a specified byte from the APP_CONFIG configuration structure to a given value.
 *
 * @param field The byte to set in the APP_CONFIG structure. For example "MyMask.v[0]"
 * @param value Contains the value to set the given byte to
 *
 * @return Returns the requested byte
 */
#define appcfgPutcAtField(field, value) EEDATA = (value); _appcfgPutc(offsetof(APP_CONFIG, field))

/**
 * Set a specified byte from the APP_CONFIG configuration structure with the data contained
 * in the EEDATA byte.
 *
 * @param offset Contains the offset in the APP_CONFIG structure of the byte to get
 *
 * @return Returns the requested byte
 */
void _appcfgPutc(BYTE offset);

/**
 * Set a specified byte in the APP_CONFIG configuration structure to a given value.
 * This function will only update the EEPROM if the given new value is different from the
 * current old value. This saves the EEPROM, and increases it's live!
 *
 * @param offset The byte to set in the APP_CONFIG structure. Use the APPCFG_X defines
 * @param value Contains the value to set the given byte to
 *
 * @return Returns the requested byte
 */
#define appcfgPutc(offset, value) EEDATA = (value); _appcfgPutc(offset)

#endif
