/*********************************************************************  
 *  Simple Network Time Protocol (SNTP) Client
 *  Adapted from SNTP Module for Microchip TCP/IP Stack
 *   -Locates an NTP Server from public site using DNS  
 *   -Requests UTC time using SNTP and updates SNTPTime structure  
 *    periodically, according to NTP_QUERY_INTERVAL value  
 *  - Reference: RFC 1305  
 *  
 *********************************************************************  
 * FileName:        SNTP.c  
 * Dependencies:    TCP/IP stack
 *  
 ********************************************************************/   
#define __SNTP_C    

#include "projdefs.h"
#include "net\dns.h"
#include "net\udp.h"    
#include "net\arptsk.h"
#include "net\tick.h"
#include "net\helpers.h"
#include "sntp\sntp.h"

#include "proto_board\leds.h"

#include <string.h>

#if defined(STACK_USE_SNTP_CLIENT)    

#define NTP_QUERY_INTERVAL      ((TICK16)3600)         // Resynchronize date/time every hour    
#define NTP_SERVER_PORT         (123ul)    
#define NTP_EPOCH               (86400ul * (365ul * 70ul + 17ul))   // 0:0:0 1/1/1970    
#define NTP_REPLY_TIMEOUT       (6ul*TICK_SECOND)                   // Wait 6 seconds for timeout    
   
#define NTP_SERVER  "fr.pool.ntp.org"    
//#define NTP_SERVER    "europe.pool.ntp.org"    
//#define NTP_SERVER    "asia.pool.ntp.org"    
//#define NTP_SERVER    "oceania.pool.ntp.org"    
//#define NTP_SERVER    "north-america.pool.ntp.org"    
//#define NTP_SERVER    "south-america.pool.ntp.org"    
//#define NTP_SERVER    "africa.pool.ntp.org"    

typedef struct _NTP_PACKET   
{   
    struct   
    {   
        BYTE mode           : 3;   
        BYTE versionNumber  : 3;   
        BYTE leapIndicator  : 2;   
    } flags;
   
    BYTE stratum;   
    char poll;   
    char precision;   
    DWORD root_delay;   
    DWORD root_dispersion;   
    DWORD ref_identifier;   
    DWORD ref_ts_secs;   
    DWORD ref_ts_fraq;   
    DWORD orig_ts_secs;   
    DWORD orig_ts_fraq;   
    DWORD recv_ts_secs;   
    DWORD recv_ts_fraq;   
    DWORD tx_ts_secs;   
    DWORD tx_ts_fraq;   
} NTP_PACKET;   
   
DWORD dwSNTPSeconds = 0;   
static DWORD dwLastUpdateTickSec = 0;   
static enum   
    {
		SM_WAIT=0,
        SM_HOME,   
        SM_NAME_RESOLVE,   
        SM_ARP_START_RESOLVE,   
        SM_ARP_RESOLVE,   
        SM_ARP_START_RESOLVE2,   
        SM_ARP_RESOLVE2,   
        SM_ARP_START_RESOLVE3,   
        SM_ARP_RESOLVE3,   
        SM_ARP_RESOLVE_FAIL,   
        SM_UDP_SEND,   
        SM_UDP_RECV,
		SM_STOP
    } SNTPState = SM_WAIT;
	
static BYTE ntp_server[32];
   
   
/*********************************************************************  
 * Function:        void SNTPClient(void)  
 *  
 * PreCondition:    None  
 *  
 * Input:           None  
 *  
 * Output:          Sets the local date and time to match remote NTP   
 *                  server value  
 *  
 * Side Effects:    None  
 *  
 * Overview:        None  
 *  
 * Note:            Require one available UDP socket resource.  The   
 *                  UDP socket is freed while SNTP module is idle.  
 ********************************************************************/   
void SNTPClient(void)   
{   
    NTP_PACKET          pkt;   
    WORD                w;
    static NODE_INFO    Server;      
    static UDP_SOCKET   MySocket;
	static DWORD        dwTimer;
	static WORD         timerReloadLow=TICK_COUNTER_LOW;
	
    switch(SNTPState)   
    {
        case SM_WAIT:
            // Requery the NTP server at reset or after a specified NTP_QUERY_INTERVAL time (ex: 10 minutes) has elapsed.    
            if((dwLastUpdateTickSec==0) ||(TickGetDiffSec(dwTimer) >= NTP_QUERY_INTERVAL)) {
                SNTPState = SM_HOME;
			}
            break;	
        case SM_HOME:
			//Get host name from EEPROM
			for(w=0;w<32;w++) ntp_server[w]=appcfgGetc(APPCFG_SNTP_NAME0+w);
			//string max length is 32 chars
			ntp_server[31]=0;
				
			// If hostname is not defined
			if (ntp_server[0]==0) {
				//client is stopped
				SNTPState = SM_STOP;
				break;
			}
            // Obtain the IP address associated with the server name
            DNSResolve(ntp_server);
			SetLeds(DNS_LED);
            dwTimer = TickGet();   
            SNTPState = SM_NAME_RESOLVE;   
            break;   
   
        case SM_NAME_RESOLVE:   
            // Wait for DNS resolution to complete    
            if(DNSIsResolved(&Server.IPAddr) == FALSE)
            {													// waiting for DNS
                if((TickGet() - dwTimer) > (5 * TICK_SECOND))    
                {												// DNS timeout
                    dwTimer = TickGet();   
                }   
                break;   
            } else ClearLeds(DNS_LED);
			// DNS complete
            SNTPState = SM_ARP_START_RESOLVE;   
            // No need to break    
   
        case SM_ARP_START_RESOLVE:   
        case SM_ARP_START_RESOLVE2:   
        case SM_ARP_START_RESOLVE3:   
            // Obtain the MAC address associated with the server's IP address     
            ARPResolve(&Server.IPAddr);   
            dwTimer = TickGet();   
            SNTPState++;   
            break;   
   
        case SM_ARP_RESOLVE:   
        case SM_ARP_RESOLVE2:   
        case SM_ARP_RESOLVE3:   
            // Wait for the MAC address to finish being obtained    
            if(!ARPIsResolved(&Server.IPAddr, &Server.MACAddr))   
            {   
                // Time out if too much time is spent in this state    
                if(TickGet() - dwTimer > 1*TICK_SECOND)   
                {   
                    // Retransmit ARP request by going to next SM_ARP_START_RESOLVE state or fail by going to SM_ARP_RESOLVE_FAIL state.    
                    SNTPState++;   
                }   
                break;   
            }   

            SNTPState = SM_UDP_SEND;   
            break;   
   
        case SM_ARP_RESOLVE_FAIL:   
            // ARP failed after 3 tries, abort and wait for next time query    
            dwTimer = TickGet();
            break;   
   
        case SM_UDP_SEND:
            // Open up the sending UDP socket    
            MySocket = UDPOpen(4020, &Server, NTP_SERVER_PORT);   
            if(MySocket == INVALID_UDP_SOCKET)   
                break;   
   
            // Make certain the socket can be written to    
            if(!UDPIsPutReady(MySocket))   
            {   
                UDPClose(MySocket);   
                break;   
            }   
   
            // Transmit a time request packet    
            memset(&pkt, 0, sizeof(pkt));   
            pkt.flags.versionNumber = 3;    // NTP Version 3    
            pkt.flags.mode = 3;             // NTP Client    
            pkt.orig_ts_secs = swapl(NTP_EPOCH);   
            UDPPutArray((BYTE*) &pkt, sizeof(pkt));    
            UDPFlush();    
               
            dwTimer = TickGet();   
            SNTPState = SM_UDP_RECV;           
            break;   
   
        case SM_UDP_RECV:   
            // Look for a response time packet    
            if(!UDPIsGetReady(MySocket))    
            {   
                if((TickGet()) - dwTimer > NTP_REPLY_TIMEOUT)   
                {   
                    // Abort the request and wait until the next timeout period    
                    UDPClose(MySocket);   
                    dwTimer = TickGet();
                    SNTPState = SM_WAIT;   
                    break;   
                }   
                break;   
            }
               
            // Get the response time packet    
            w = UDPGetArray((BYTE*) &pkt, sizeof(pkt));   
            UDPClose(MySocket);      
            SNTPState = SM_WAIT;   
   
            // Validate packet size    
            if(w != sizeof(pkt))    
            {   
                break;     
            }   

            // Set out local time to match the returned time     
			//use dwTimer for save temporary SNTP value
            dwTimer = swapl(pkt.tx_ts_secs) - NTP_EPOCH;   
            // Do rounding.  If the partial seconds is > 0.5 then add 1 to the seconds count.    
            if(((BYTE*)&pkt.tx_ts_fraq)[0] & 0x80)   
                  dwTimer++;
			
			//if it's not the first ntp time setting
			if (dwLastUpdateTickSec!=0) {
				//get delta between new sntp time and internal time. Delta is truncated to + or - 60s
				if ( dwTimer >= SNTPGetUTCSeconds()+60) dwTimer= 60;
				else if (SNTPGetUTCSeconds() >= dwTimer+60) dwTimer= -60;
				else dwTimer-=SNTPGetUTCSeconds();
				//timer value update is 7 time time delta in seconds
				dwTimer=(long int)7*(long int)(dwTimer);
				
				//update  value of reload timer
				timerReloadLow+=(int)dwTimer;
				if ( (int)timerReloadLow >= 0xFF) timerReloadLow=0xFF;
				else if ( (int)timerReloadLow <=0) timerReloadLow=0;
				//Add temporary  the same value to timerReloadLow and load tickCountLow with the result to resynchronize internal time and sntp  time
				if ((int)(timerReloadLow+dwTimer) >= 0xFF) tickCounterLow=0xFF;
				else if ((int)(timerReloadLow+dwTimer) <= 0) tickCounterLow=0;
				else tickCounterLow=(BYTE)(timerReloadLow+dwTimer);
			} else {
				//save SNTP Time at first SNTP Time request only
				dwSNTPSeconds=dwTimer;
				dwLastUpdateTickSec=TickGetSec();
			} 
			//Save TickSec at time update
			dwTimer=TickGetSec();
            break;   
    }   
}   
   
   
/*********************************************************************  
 * Function:        DWORD SNTPGetUTCSeconds(void)  
 *  
 * PreCondition:    None  
 *  
 * Input:           None  
 *  
 * Output:          DWORD indicating the number of seconds since   
 *                  00:00:00 1970  
 *  
 * Side Effects:    None  
 *  
 * Overview:        None  
 *  
 * Note:            Use this time value for absolute time stamping.  
 *  
 *                  Do not use this API for time difference   
 *                  measurements. Since this value depends on what   
 *                  the remote NTP server periodically provides us   
 *                  with, one call to SNTPGetUTCSeconds() could   
 *                  return a higher number than a second call to   
 *                  SNTPGetUTCSeconds() made at a future time.  Use   
 *                  the TickGet*() API for reliable time difference   
 *                  calculations.  
 ********************************************************************/   
DWORD SNTPGetUTCSeconds(void)   
{   
    return (dwSNTPSeconds+TickGetDiff32bitSec(dwLastUpdateTickSec));   
}   


void StartSNTP() {
	dwLastUpdateTickSec=0;
	dwSNTPSeconds=0;
	SNTPState=SM_WAIT;
}

BYTE StateSNTP() {
	return (BYTE) SNTPState;
}

#endif  //if defined(STACK_USE_SNTP_CLIENT)   
